package agent.newframe;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;

import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import agentspace.Context;
import agentspace.InvalidURLException;
import agentspace.Agent;
import agentspace.Mobile;
import agentspace.Duplicatable;
import agentspace.Preservable;
import agentspace.event.AgentEvent;

public class TestWindow implements Agent, Mobile, Preservable, Duplicatable {
	transient AgentFrame frame = null;
	JTextArea area;

	public static void main(String[] args) {
		TestWindow test = new TestWindow();
		test.create(null, null);
	}

	
	class AgentFrame extends JFrame implements Serializable {
		Context context;
		ActionListener listener;
		JPanel panel;
		JButton moveButton;
		JButton saveButton;
		JButton copyButton;
		JButton stopButton;
		
		public AgentFrame(JTextArea area, Context context) {
			super("Window Agent");
			this.context = context;
			listener = new AgentAction(this);
			panel = new JPanel();

			moveButton = new JButton("Move");
			moveButton.setActionCommand("Move");
			moveButton.addActionListener(listener);
			panel.add(moveButton);
			
			saveButton = new JButton("Save");
			saveButton.setActionCommand("Save");
			saveButton.addActionListener(listener);
			panel.add(saveButton);
			
			copyButton = new JButton("Copy");
			copyButton.setActionCommand("Copy");
			copyButton.addActionListener(listener);
			panel.add(copyButton);
			
			stopButton = new JButton("Stop");
			stopButton.addActionListener(listener);
			stopButton.setActionCommand("Stop");
			panel.add(stopButton);
			
			area.setVisible(true);
			getContentPane().setLayout(new BorderLayout());
			getContentPane().add(panel, BorderLayout.NORTH);
			getContentPane().add(area, BorderLayout.CENTER);
			setSize(400, 300);
		}

	
		class AgentAction implements ActionListener, Serializable {
			JFrame frame;
			public AgentAction(JFrame frame) {
				this.frame = frame;
			}
			public void actionPerformed(ActionEvent evt) {
				String action = evt.getActionCommand();
				if (action.equals("Move")) {
					System.out.println("TestWindow: evt = "+evt);
					try {
						URL url = new URL("matp://localhost:7000");
						AgentDestinationDialog dialog = new AgentDestinationDialog(frame, url);
						if (dialog.getURL() != null) {
							context.move(dialog.getURL());
						}
					} catch (MalformedURLException e) {
						e.printStackTrace();
					} catch (IOException e) {
						e.printStackTrace();
					} catch (InvalidURLException e) {
						e.printStackTrace();
					}
				} 
				else if (action.equals("Save")) {
					try {
						String filename = null; 
						JFileChooser chooser = new JFileChooser();
						int state = chooser.showSaveDialog(null);
						File file = chooser.getSelectedFile();
						if (file != null && state == JFileChooser.APPROVE_OPTION) {
							filename = file.getPath();
							System.out.println("URL = "+filename);
							context.store(filename);
						}
					} 
					catch (Exception e) {
						e.printStackTrace();
					}
				} 
				else if (action.equals("Copy")) {
					try {
						context.duplicate();
					} 
					catch (Exception e) {
						e.printStackTrace();
					}
				}
				else if (action.equals("Stop")) {
					System.out.println("TestWindow: evt = "+evt);
					try {
						context.terminate();
					} 
					catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		}
		
		class AgentDestinationDialog {
			public URL url = null;
			public AgentDestinationDialog(JFrame frame, URL url) {
				JTextField textField = new JTextField(30);
				textField.setText(url.toString());
				textField.setBorder(new TitledBorder("go to URL:"));
				Object[] obj = {textField};
				int ans = JOptionPane.showConfirmDialog(frame.getContentPane(), obj, "Dialog", JOptionPane.OK_CANCEL_OPTION);
				if (ans == JOptionPane.YES_OPTION) {
					try {
						url = new URL(textField.getText());
						System.out.println("AgentDestinationDialog: url = "+url);
					} 
					catch (MalformedURLException e) {
						e.printStackTrace();
						url = null;
					}
				} 
				else if (ans == JOptionPane.CANCEL_OPTION) {
					url = null;
				}
				this.url = url;
			}
			public URL getURL() {
				return url;
			}
		}
	}
	
	
	public TestWindow() {
		System.out.println("Test: an instance created.");
	}
		
	/* 
	 * @see agentspace.SerialiableAgent#create(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void create(AgentEvent evt, Context context) {
		System.out.println("Test: method create() called");
		area = new JTextArea(80, 20);
		area.setText("Input some text here.\n");
		frame = new AgentFrame(area, context);
		frame.setVisible(true);
	}
	/* 
	 * @see agentspace.SerialiableAgent#destroy(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void destroy(AgentEvent evt, Context context) {
		System.out.println("Test: method destroy() called");
		frame.setVisible(false);
	}
	/* 
	 * @see agentspace.SerialiableAgent#suspend(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void suspend(AgentEvent evt, Context context) {
		System.out.println("Test: method suspend() called");
		frame.setVisible(false);
	}

	/* 
	 * @see agentspace.SerialiableAgent#resume(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void resume(AgentEvent evt, Context context) {
		System.out.println("Test: method resume() called");
		frame = new AgentFrame(area, context);
		frame.setVisible(true);
	}
	/* 
	 * @see agentspace.MobileAgent#arrive(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void arrive(AgentEvent evt, Context context) {
		System.out.println("Test: method arrive() called");
		frame = new AgentFrame(area, context);
		frame.setVisible(true);
	}
	/* 
	 * @see agentspace.MobileAgent#leave(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void leave(AgentEvent evt, Context context) {
		System.out.println("Test: method leave() called");
		frame.setVisible(false);
	}
	/*
	 * @see agentspace.DuplicatableAgent#duplicate(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void duplicate(AgentEvent evt, Context context) {
		System.out.println("Test: method duplicate() called");
		frame.setVisible(false);
	}

	/*
	 * @see agentspace.DuplicatableAgent#parent(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void parent(AgentEvent evt, Context context) {
		System.out.println("Test: method parent() called");
		frame = new AgentFrame(area, context);
		frame.setVisible(true);
	}

	/*
	 * @see agentspace.DuplicatableAgent#child(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void child(AgentEvent evt, Context context) {
		System.out.println("Test: method child() called");
		frame = new AgentFrame(area, context);
		frame.setVisible(true);
	}
}
