/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/4
 */

package agentspace;

import java.io.IOException;
import java.net.InetAddress;
import java.net.URL;
import java.net.UnknownHostException;

import agentspace.network.AgentSocketServer;
import agentspace.AgentManager;

public class AgentServer {
	final static public int SERVER_PORT = 7000;
	static int port;

	static {
		URL.setURLStreamHandlerFactory(new agentspace.network.AgentURLStreamHandlerFactory());
	}
	
	public static void main(String[] args) {
		String command = null;
		port = SERVER_PORT;
		for (int i = 0; i < args.length; i++) {
			if ((args[i].equals("-port") || args[i].equals("-p"))
					&& (i + 1 < args.length)) {
				try {
					port = new Integer(args[++i]).intValue();
				} 
				catch (NumberFormatException ex) {
					//                    System.out.println("invalid port number [" + args[i] + "]");
					System.exit(1);
				}
			} 
			else if (args[i].equals("-cmd") || args[i].equals("-command")) {
				command = "";
				for (int j = 1; j < args.length - i; j++) {
					command = command + " " + args[i + j];
				}
				break;
			} 
		}
		AgentServer as = new AgentServer(command, port);
	}
	
	AgentServer(String command, int port) {
		System.out.println("AgentServer:"+port);
		if (command == null || command.length() == 0) {
			try {
				AgentManager manager = AgentManager.instance();
				AgentSocketServer server = new AgentSocketServer(manager, port);
				server.start();
			}
			catch (IOException ex) {
				ex.printStackTrace();                
			}
		}
	}
	
	static public int getDefaultPort() {
		return port;
	}
	
	static public String getLocalHost() throws UnknownHostException {
		InetAddress addr = InetAddress.getLocalHost();
		return addr.getHostAddress();
	}
}