/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/6
 */

package agentspace.network;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Hashtable;

import agentspace.network.protocol.RequestFactory;
import agentspace.network.protocol.Response;
import agentspace.network.protocol.ResponseParser;

public class SocketClient {
	static Hashtable handlers = new Hashtable();
	
	public static URLSessionHandler createURLSessionHandler(String protocol) {
		URLSessionHandler handler = (URLSessionHandler)handlers.get(protocol);
		if (handler == null) {
			String className = "agentspace.network.protocol." + protocol.toLowerCase() + ".URLSessionHandler";
			try {
				handler = (URLSessionHandler)Class.forName(className).newInstance();
			}
			catch (ClassNotFoundException e) {
				handler = null;
//				e.printStackTrace();
			}
			catch (Exception e) {
				handler = null;
				e.printStackTrace();
			}
			if (handler != null) {
				handlers.put(protocol, handler);
			}
		}
		return handler;
	}

	public static void isValidURL(URL url) throws IOException {
		if (url == null) {
			throw new IOException("Illegal URL = "+url);
		}
		String protocol = url.getProtocol();
		URLConnection connection = url.openConnection();
		OutputStream out = connection.getOutputStream();
		InputStream in = connection.getInputStream();
		if (protocol.equalsIgnoreCase("MATP")) {
			RequestFactory request = new RequestFactory();
			request.setProtocol("MATP");
			request.setVersion("1.0");
			request.setMethod("HELLO");
			request.setURI("/");
			out.write(request.toByteArray());
			out.flush();
			Response response = new ResponseParser(in);
//			System.out.println("SocketClient: receiving "+response+" from "+url);
			if (response.getCode() < 200 || response.getCode() >= 300) { 
				throw new IOException("Receiving error code from server");
			}
		}
		else if (protocol.equalsIgnoreCase("HTTP")) {
			RequestFactory request = new RequestFactory();
			request.setProtocol("HTTP");
			request.setVersion("1.0");
			request.setMethod("HELLO");
			request.setURI("/");
			out.write(request.toByteArray());
			out.flush();
			Response response = new ResponseParser(in);
//			System.out.println("SocketClient: receiving "+response+" from "+url);
			if (response.getCode() < 200 || response.getCode() >= 300) { 
				throw new IOException("Receiving error code from server");
			}
		}
		else {
			throw new IOException("No such protocol = "+url.getProtocol());
		}
		if (out != null) {
			try {
				out.close();
			}
			catch (IOException ex) {}
		}				
	}
	
	public static void moveContent(URL url, byte[] content) throws IOException {
		try {
			if (url == null) {
				throw new IOException("Illegal URL = "+url);
			}
			String protocol = url.getProtocol();
			if (protocol == null) {
				
			}
			URLConnection connection = url.openConnection();
			OutputStream out = connection.getOutputStream();
			InputStream in = connection.getInputStream();
			if (protocol.equalsIgnoreCase("MATP")) {
				RequestFactory request = new RequestFactory();
				request.setProtocol("MATP");
				request.setVersion("1.0");
				request.setMethod("MOVE");
//				request.setURI(url.getFile());
				request.setURI("/");
				request.setContent(content);
				out.write(request.toByteArray());
				out.flush();
//				System.out.println("SocketClient: sending "+content.length+" to "+url);
				Response response = new ResponseParser(in);
//				System.out.println("SocketClient: receiving "+response+" from "+url);
//				System.err.println("Response = "+response);
				if (response.getCode() < 200 || response.getCode() >= 300) { 
					throw new IOException("I/O Exception");
				}
			}
			else if (protocol.equalsIgnoreCase("HTTP")) {
				RequestFactory request = new RequestFactory();
				request.setProtocol("HTTP");
				request.setVersion("1.0");
				request.setMethod("PUT");
				request.setURI(url.getFile());
				request.setContent(content);
				out.write(request.toByteArray());
				out.flush();
				Response response = new ResponseParser(in);
//				System.err.println("Response = "+response);
				if (response.getCode() < 200 || response.getCode() >= 300) { 
					throw new IOException("I/O Exception");
				}
			}
			else {
			
			}
			out.close();
		} 
		catch (MalformedURLException e) {
			e.printStackTrace();
		} 
	}

	public static byte[] getContent(URL url) throws IOException {
		byte[] data = null;
		try {
			if (url == null) {
				throw new IOException("Illegal URL = "+url);
			}
			String protocol = url.getProtocol();
			if (protocol == null) {
				throw new IOException("Unknown protocol");
			}
			URLConnection connection = url.openConnection();
			OutputStream out = connection.getOutputStream();
			InputStream in = connection.getInputStream();
			if (protocol.equalsIgnoreCase("MATP")) {
				RequestFactory request = new RequestFactory();
				request.setProtocol("MATP");
				request.setVersion("1.0");
				request.setMethod("FETCH");
				request.setURI(url.getFile());
				out.write(request.toByteArray());
				out.flush();
				Response response = new ResponseParser(in);
				System.err.println("Response = "+response);
				if (response.getCode() < 200 || response.getCode() >= 300) { 
					throw new IOException("I/O Exception");
				}
				data = response.getContent();
			}
			else if (protocol.equalsIgnoreCase("HTTP")) {
				RequestFactory request = new RequestFactory();
				request.setProtocol("HTTP");
				request.setVersion("1.0");
				request.setMethod("PUT");
				request.setURI(url.getFile());
				out.write(request.toByteArray());
				out.flush();
				Response response = new ResponseParser(in);
				System.err.println("Response = "+response);
				if (response.getCode() < 200 || response.getCode() >= 300) { 
					throw new IOException("I/O Exception");
				}
				data = response.getContent();
			}
			else {
				throw new IOException("No such protocol = "+url.getProtocol());
			}
			if (out != null) {
				out.close();
			}				
		} 
		catch (MalformedURLException e) {
			e.printStackTrace();
		} 
		return data;
	}
}
