/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/11
 */

package agentspace.registry;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import agentspace.AgentIdentifier;
import agentspace.NoSuchAgentException;
import agentspace.runtime.AgentContainer;

public class AgentRegistry {
	protected Hashtable table = null;
	protected Hashtable containers = null;

	public AgentRegistry() {
		table = new Hashtable();
	}

	public void add(AgentContainer ac) {
		table.put(ac.getIdentifier(), ac);
	}
	
	public void remove(AgentContainer ac) throws NotBoundException {
		if (table.remove(ac.getIdentifier()) == null) {
			throw new NotBoundException();
		}
	}
	
	public AgentContainer[] lookup(String name) throws NotBoundException {
		List list = new Vector();
		Enumeration enum = table.elements();
		while (enum.hasMoreElements()) {
			Object obj = enum.nextElement();
			if (obj instanceof AgentContainer) {
				AgentContainer ac = (AgentContainer)obj;
				if (name.equals(ac.getName())) {
					list.add(ac);
				}
			}
		}
		if (list.isEmpty()) {
			throw new NotBoundException("No such element named"+name);
		}
		return (AgentContainer[])list.toArray();
	}
	
	public AgentContainer getContainer(AgentIdentifier aid) 
	throws NoSuchAgentException {
		Object obj = table.get(aid);
		if (obj != null && obj instanceof AgentContainer) {
			return (AgentContainer)obj;
		}
		else {
			throw new NoSuchAgentException(aid.toString());
		}
	}
	
	public int size() {
		return table.size();
	}
	
	public AgentContainer[] elements() {
		AgentContainer[] acs = new AgentContainer[size()];
		Enumeration enum = table.elements();
		for (int i = 0 ; enum.hasMoreElements() && i < size() ; i++ ) {
			Object obj = enum.nextElement();
			if (obj instanceof AgentContainer) {
				acs[i] = (AgentContainer)obj;
			}
		}
		if (size() == 0) {
			return null;
		}
		else {
			return acs;
		}
	}
} 
