/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/5
 */

package agentspace.runtime;

import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.Vector;

public class AgentQueue {
	protected Vector list = null;

	public synchronized void enqueue(Object item) {
		if (list == null) {
			list = new Vector();
		}
		list.addElement(item);
		if (size() == 1) {
			notifyAll();
		}
	}

	public synchronized Object dequeue() throws InterruptedException {
		Object item = null;
		if (list == null) {
			list = new Vector();
		}
		while (list.size() <= 0) {
			wait();
		}
		item = list.elementAt(0);
		list.removeElementAt(0);
		return item;
	}

	public synchronized void enqueueAtTop(Object item) {
		if (list == null) {
			list = new Vector();
		}
		list.insertElementAt(item, 0);
		if (list.size() == 1) {
			notifyAll();
		}
	}

	public synchronized void remove(Object item) throws NoSuchElementException {
		if (list == null) {
			list = new Vector();
		}
		if (!(list.removeElement(item))) {
			throw new NoSuchElementException(item.toString());
		}
	}

	public void clear() {
		if (list != null) {
			list.removeAllElements();
			list = null;
		}
	}

	public synchronized boolean isEmpty() {
		if (list == null) {
			return true;
		}
		if (list.size() == 0) {
			return true;
		}
		return false;
	}

	public synchronized int size() {
		if (list == null) {
			return 0;
		}
		return list.size();
	}

	public Enumeration elements() {
		if (list == null) {
			list = new Vector();
		}
		return list.elements();
	}

	public Object elementAt(int i) throws NoSuchElementException {
		if (list == null || list.isEmpty()) {
			throw new NoSuchElementException("No Item at " + i);
		}
		return list.elementAt(i);
	}

	public String toString() {
		String dump = "";
		if (list != null) {
			for (int i = 0; i < list.size(); i++) {
				dump = dump + " " + list.elementAt(i);
			}
		}
		return dump;
	}

	public int search(Object item) {
		int i = list.indexOf(item, 0);
		if (i >= 0) {
			return i;
		}
		return -1;
	}
}